/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;
import org.ow2.jasmine.rules.osgi.impl.OSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.impl.OSGiDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.impl.PackageAdderListener;


/**
 * Start and stop the Drools working memory.
 * @author Loris Bouzonnet
 */
public class Activator implements BundleActivator {

    /**
     * Logger.
     */
    private static Logger logger = Logger
            .getLogger(Activator.class.getName());

    /**
     * Keep a trace of the registration service in order to unregister it before stopping.
     */
    private ServiceRegistration serviceRegistration;

    /**
     * The Drools working memory pool
     */
    private IDroolsWorkingMemoryPool droolsWorkingMemoryPool;

    /**
     * Listener to dynamically add and remove rules packages.
     */
    private ServiceListener packageAdderListener;

    /**
     * Start the working memory and register it in the given {@link BundleContext}.
     * @param context the context to register the working memory
     * @throws Exception if the working memory cannot be started
     */
    public void start(final BundleContext context) throws Exception {

        // Retrieve the already registered rules packages

        //// Retrieve the services references
        ServiceReference[] packageAdderRefs =
            context.getAllServiceReferences(IPackageAdder.class.getName(), null);

        //// A hashmap to associate targeted working memories names
        //// to package adders
        HashMap<String,HashMap<ServiceReference, IPackageAdder>> packageAdders =
            new HashMap<String,HashMap<ServiceReference, IPackageAdder>>();

        //// Populate the previous hashmap
        if(packageAdderRefs != null) {
            for(ServiceReference packageAdderRef : packageAdderRefs) {

                // Get package adder targeted working memory name
                String workingMemoryName = ((IPackageAdder) context.getService(packageAdderRef)).getWorkingMemoryName();

                // Add the package adders in the hashmap
                if(packageAdders.containsKey(workingMemoryName)) {
                    packageAdders.get(workingMemoryName).put(
                            packageAdderRef,
                            (IPackageAdder) context.getService(packageAdderRef));
                } else {
                    HashMap<ServiceReference, IPackageAdder> newPackageAdders =
                        new HashMap<ServiceReference, IPackageAdder>();
                    newPackageAdders.put(
                            packageAdderRef,
                            (IPackageAdder) context.getService(packageAdderRef));
                    packageAdders.put(
                            workingMemoryName,
                            (HashMap<ServiceReference, IPackageAdder>) newPackageAdders);
                }
            }
        }

        // Init the working memories pool
        droolsWorkingMemoryPool = new OSGiDroolsWorkingMemoryPool(context);

        // Init each working memory
        for(String workingMemoryName: packageAdders.keySet()){
            logger.log(Level.INFO,"Creating and starting WorkingMemory \"" + workingMemoryName + "\"");

            // Create the new working memory
            OSGiDroolsWorkingMemory newDroolsWorkingMemory =
                new OSGiDroolsWorkingMemory(context,
                    (Map<ServiceReference, IPackageAdder>) packageAdders.get(workingMemoryName));

            // Add it to the pool
            droolsWorkingMemoryPool.add(
                    workingMemoryName,
                    newDroolsWorkingMemory);

            // Start the working memory
            newDroolsWorkingMemory.start();

        }

        // Add a listener for adding new packages to the working memories
        packageAdderListener = new PackageAdderListener(context, (IOSGiDroolsWorkingMemoryPool) droolsWorkingMemoryPool);
        context.addServiceListener(packageAdderListener, "(objectClass=" + IPackageAdder.class.getName() + ")");

        // Register the working memory pool service
        serviceRegistration =
            context.registerService(
                    IDroolsWorkingMemoryPool.class.getName(),
                    droolsWorkingMemoryPool,
                    null);

    }

    /**
     * Unregister the working memory of the given {@link BundleContext} and stop it.
     * @param context the context to unregister the working memory
     */
    public void stop(final BundleContext context) {

        // Unregister the working memory pool service
        if(serviceRegistration != null) {
            serviceRegistration.unregister();
        }

        // Remove the package listener
        if(packageAdderListener != null) {
            context.removeServiceListener(packageAdderListener);
        }

        // Stop all the working memories
        droolsWorkingMemoryPool.stopAll();

    }

}
