/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Logger;

import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.conf.EventProcessingOption;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemory;
import org.ow2.jasmine.rules.api.DroolsWorkingMemoryState;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

import static org.drools.KnowledgeBaseFactory.newKnowledgeBaseConfiguration;
import static org.ow2.jasmine.rules.api.DroolsWorkingMemoryState.*;

/**
 * A service implementation for OSGi to manage the Drools working memory.
 *
 * @author Loris Bouzonnet
 */
public class OSGiDroolsWorkingMemory extends AbstractDroolsWorkingMemory
                                     implements IOSGiDroolsWorkingMemory {

    /**
     * Logger
     */
    private static Logger logger = Logger
            .getLogger(OSGiDroolsWorkingMemory.class.getName());

    /**
     * Delegate class loading to a set of bundles.
     */
    private final MultiBundleClassLoader multiBundleClassLoader;

    /**
     * Used package adders.
     */
    protected final Map<ServiceReference, IPackageAdder> packageAdders;


    // ------------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------------

    /**
     * Create a manager of working memory that use OSGi to add (and remove)
     * packages.
     * @param context
     *
     * @param externalLoader
     *            a classloader to use for adding package
     * @param packageAdders
     *            initial package adders
     * @throws Exception
     */
    public OSGiDroolsWorkingMemory(final BundleContext context,
                                   final Map<ServiceReference, IPackageAdder> packageAdders)
            throws Exception {

        // Call super class constructor
        super(logger);

        // Initialize properties
        this.packageAdders = packageAdders;
        this.multiBundleClassLoader = new MultiBundleClassLoader(getClass().getClassLoader());
        multiBundleClassLoader.addBundleClassLoader(context.getBundle());

        initKnowledgeBuilderConfiguration(multiBundleClassLoader);
    }


    // ------------------------------------------------------------------------
    // Implementation of the IDroolsWorkingMemory interface
    // ------------------------------------------------------------------------


    public void start() throws Exception {
        // Check if some rules packages are ready to be registered
        if (!packageAdders.isEmpty()) {
            addPackages(packageAdders.values());
        } else {
            state = IDLE;
            debug("WorkingMemory[<name>] No Packages to use yet, going to state {0}", state);
        }
    }

    /**
     * Add a rules package by using the given package adder.
     *
     * @param serviceReference
     *            a reference to the given package adder
     * @param packageAdder
     *            a package adder
     * @throws Exception
     *             if a package cannot be built or added to the rule base
     */
    public void addPackage(final ServiceReference serviceReference,
                           final IPackageAdder packageAdder) throws Exception {

        if (packageAdders.put(serviceReference, packageAdder) == null
            && state != STOPPED) {
            debug("WorkingMemory[<name>] Installing Packages from bundle {0}",
                  serviceReference.getBundle());
            addPackages(Collections.singletonList(packageAdder));
        } else {
            debug("WorkingMemory[<name>] Not installing Packages from bundle {0}" +
                  " (duplicate or STOPPED state) [current:{1}]",
                  serviceReference.getBundle(), state);
        }
    }

    /**
     * Remove a package adder for a given service reference.
     *
     * @param serviceReference
     *            a reference to a given package adder
     * @throws Exception
     *             if the working memory cannot be restarted
     */
    public void removePackage(final ServiceReference serviceReference) throws Exception {

        debug("WorkingMemory[<name>] Removing Packages from bundle {0}",
              serviceReference.getBundle());

        // Spot the corresponding packageAdder
        IPackageAdder packageAdder = packageAdders.get(serviceReference);

        // If found, remove it
        if (packageAdder != null) {

            // Remove it from the registered package adders
            packageAdders.remove(serviceReference);

            // Remove its classloader from the composite class loader
            multiBundleClassLoader.removeBundleClassLoader(packageAdder.getBundle());

            // If this is a DRL package adder, we know the package name and can
            // directly remove it form the RuleBase
            if (packageAdder instanceof DrlPackageAdder) {
                String name = ((DrlPackageAdder) packageAdder).getPackageName();
                knowledgeBase.removeKnowledgePackage(name);
                debug("WorkingMemory[<name>] Removing Package[{0}] from the KnowledgeBase", name);
            } // And if not ? the package stays there

            // Update the rule engine state if needed
            if (packageAdders.size() < 1) {
                // We removed the last adder, so the list is empty
                debug("WorkingMemory[<name>] No more Packages in the memory");

                // Update our inner state
                if (state != STOPPED) {
                    // (IDLE or RUNNING) -> IDLE
                    stop(IDLE);
                } else {
                    // (STOPPED) -> STOPPED !? Weird
                    // That should means that we're already STOPPED
                    stop(STOPPED);
                }
            } else {
                // There are still some package adders here
                // FIXME we have to restart both the KnowledgeBase and the KnowledgeSession ...
                debug("WorkingMemory[<name>] Still have some Packages in the memory");

                // Update our inner state
                if (state != STOPPED) {
                    // (IDLE or RUNNING) -> IDLE
                    stop(IDLE);
                } else {
                    // (STOPPED) -> STOPPED !? Weird
                    // That should means that we're already STOPPED
                    stop(STOPPED);
                }

                // Reset the instances
                // Next time, a session will be recreated
                knowledgeBase = null;
                knowledgeSession = null;
                debug("WorkingMemory[<name>] Base & Sessions nullified");

                // And restart it
                if (!packageAdders.isEmpty()) {
                    debug("WorkingMemory[<name>] Re-install available Packages");
                    addPackages(packageAdders.values());
                }
            }
        }
    }


    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * Add some rules packages by using the given package adders.
     *
     * @param packageAdders
     *            a collection of package adders
     * @throws Exception
     *             if a package cannot be built or added to the rule base
     */
    private void addPackages(final Collection<IPackageAdder> packageAdders) throws Exception {

        ClassLoader lastClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getClass().getClassLoader());
        try {
            if (knowledgeBase == null) {

                // Create a new KnowledgeBase (hosting the rules)
                // ----------------------------------------------------

                // Generate a default configuration
                // Important: use our MultiClassLoader so that class space accessible
                // to the base can be adapted without re-creating a KnowledgeBase
                KnowledgeBaseConfiguration kbaseConfig = newKnowledgeBaseConfiguration(null, multiBundleClassLoader);
                // Always use the 'stream' mode of Drools
                kbaseConfig.setOption(EventProcessingOption.STREAM);
                // Instantiate the base
                knowledgeBase = KnowledgeBaseFactory.newKnowledgeBase(kbaseConfig);

                debug("WorkingMemory[<name>] Creating KnowledgeBase {0}", knowledgeBase);
            }

            for (IPackageAdder packageAdder : packageAdders) {

                debug("WorkingMemory[<name>] Adding Package from Bundle {0}", packageAdder.getBundle());

                // Use a new package builder for each package adder, since
                // bundles can define different packages
                KnowledgeBuilder packageBuilder = newKnowledgeBuilder();
                multiBundleClassLoader.addBundleClassLoader(packageAdder.getBundle());

                // This is where the real Package installation in the base happen
                packageAdder.addPackage(packageBuilder, knowledgeBase);

                if(knowledgeSession != null) {

                    debug("WorkingMemory[<name>] Adapting existing KnowledgeSession");
                    // Adapt the existing session (if any)
                    Map<String, Object> globals = packageAdder.getGlobals();
                    if (globals != null) {
                        for(Entry<String, Object> entry : globals.entrySet()) {
                            knowledgeSession.setGlobal(entry.getKey(), entry.getValue());
                        }
                    }
                }
            }

            if (knowledgeSession == null) {

                debug("WorkingMemory[<name>] Creating new KnowledgeSession");

                // Create and init a new WorkingMemory
                createWorkingMemory();

                // Adapt the session with the globals declared in the adders
                for (IPackageAdder packageAdder : packageAdders) {
                    Map<String, Object> globals = packageAdder.getGlobals();
                    if(globals != null) {
                        for(Entry<String, Object> entry : globals.entrySet()) {
                            knowledgeSession.setGlobal(entry.getKey(), entry.getValue());
                        }
                    }
                }

                // An executor should only be created once (with the session)
                startRuleExecutor();
            }
        } finally {
            Thread.currentThread().setContextClassLoader(lastClassLoader);
        }
    }

}
