/**
 * JASMINe
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemoryPool;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 * Listener that add rules package into the working memory when an instance of
 * service {@link IPackageAdder} is registered.
 * This occurs when a bundle, to which its activator implements
 * {@link RulesActivator}, is started.
 *
 * @author Loris Bouzonnet
 */
public class PackageAdderListener implements ServiceListener {

    // -------------------------------------------------------------------------
    // Variables
    // -------------------------------------------------------------------------

    /**
     * Logger
     */
    private static Logger logger = Logger
            .getLogger(PackageAdderListener.class.getName());

    /**
     * Working memory to add rules package.
     */
    private final IOSGiDroolsWorkingMemoryPool droolsWorkingMemoryPool;

    /**
     * OSGi bundle context to retrieve the incoming {@link IPackageAdder}.
     */
    private final BundleContext context;


    // -------------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------------

    /**
     * Construct a new listener for the given context and working memory pool.
     * @param context OSGi bundle context
     * @param droolsWorkingMemory working memory to add rules package
     */
    public PackageAdderListener(
            final BundleContext context,
            final IOSGiDroolsWorkingMemoryPool droolsWorkingMemoryPool) {
        this.context = context;
        this.droolsWorkingMemoryPool = droolsWorkingMemoryPool;
    }


    // -------------------------------------------------------------------------
    // Implementation of the ServiceListener interface
    // -------------------------------------------------------------------------

    /**
     * Triggered when the OSGi service state changed
     */
    public void serviceChanged(final ServiceEvent event) {

        // Get the package adder
        ServiceReference ref = event.getServiceReference();
        IPackageAdder packageAdder =
            (IPackageAdder) context.getService(ref);

        // Get package adder targeted working memory name
        String workingMemoryName = packageAdder.getWorkingMemoryName();

        // React on REGISTERED|UNREGISTERING events
        switch(event.getType()) {
            case ServiceEvent.REGISTERED:
                try {

                    logger.log(Level.INFO,"New rules package for the knowledge session \""
                            + workingMemoryName + "\"");

                    // Get (or create if needed) the corresponding working memory
                    IOSGiDroolsWorkingMemory theWorkingMemory;
                    try {
                        // FIXME Automatically creating the WorkingMemory when
                        // needed using getOrCreate(). May be an unwanted behavior.
                        theWorkingMemory =
                            (IOSGiDroolsWorkingMemory) ((IDroolsWorkingMemoryPool) droolsWorkingMemoryPool).getOrCreate(
                                    workingMemoryName);
                    } catch (Exception e){
                        logger.log(Level.INFO,"Could not get or create a the knowledge session " +
                                "named \"" + workingMemoryName + "\": " + e);
                        return;
                    }

                    // Add the new package
                    theWorkingMemory.addPackage(ref, packageAdder);

                } catch (Exception e) {
                    logger.log(Level.SEVERE, "Error when adding packages for the bundle "
                            + packageAdder.getBundle().getBundleId(), e);
                }
                break;
            case ServiceEvent.UNREGISTERING:
                try {

                     logger.log(Level.INFO,"Remove a rule package from" +
                            " the knowledge session \"" + workingMemoryName + "\"");

                    // Get the corresponding working memory
                    IOSGiDroolsWorkingMemory theWorkingMemory =
                        (IOSGiDroolsWorkingMemory) ((IDroolsWorkingMemoryPool) droolsWorkingMemoryPool).get(workingMemoryName);

                    // Remove the package
                    theWorkingMemory.removePackage(ref);

                } catch (Exception e) {
                    logger.log(Level.SEVERE, "Error when removing packages for the bundle "
                            + packageAdder.getBundle().getBundleId(), e);
                }
        }

    }

}
