/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.manager.model;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlTransient;

import org.ow2.jasmine.rules.manager.service.IDroolsManager;

/**
 * The base implementation of any kind of rule package.
 *
 * Defines the common fields for all rule packages types. Uses JAXB annotations for XML
 * marshaling/unmarshaling (see {@link IDroolsManager#exportRulePackageToXML(IRulePackage)}).
 *
 * @author Julien Nicoulaud
 */
public abstract class AbstractRulePackage implements IRulePackage {

    // =========================================================================
    // Properties
    // =========================================================================

    /**
     * Class version ID.
     */
    private static final long serialVersionUID = 8496763332714491590L;

    /**
     * The unique system name of the rule package. Identifies the object, must be
     * different for each rule package. Automatically generated from the rule
     * package "display" name.
     * @example "org.ow2.jasmine.monitoring.rules.example.ExamplePackage"
     */
    @XmlTransient
    public String systemName;

    /**
     * The previous systemName of the package in the case where it
     * was modified. Used to identify the old rule package.
     */
    @XmlTransient
    public String previousSystemName = null;

    /**
     * The displayed name of the rule package. This is the name that is choosed by
     * the user. Used to generate automatically the systemName property.
     * @example "Example Package"
     */
    @XmlTransient
    public String displayName;

    /**
     * The name of the rule engine instance on which the rule is executed.
     */
    @XmlTransient
    public String ruleEngineInstanceName;

    /**
     * Allowed states.
     */
    public static String DEACTIVATED = "deactivated";
    public static String ACTIVATED = "activated";
    public static String ACTIVATION_REQUESTED = "activationRequested";
    public static String DEACTIVATION_REQUESTED = "deactivationRequested";

    /**
     * The state of the rule package. Values can be:
     *   - Activated
     *   - Deactivated
     *   - Activation requested
     *   - Deactivation requested
     */
    @XmlTransient
    public String state = DEACTIVATED;

    /**
     * The description of the rule package.
     */
    @XmlTransient
    public String description;


    // =========================================================================
    // Getters and setters
    // =========================================================================

    /**
     * Get the package name.
     */
    public void setSystemName(final String value)
    {
        systemName = value;
    }

    /**
     * Set the package name.
     */
    @XmlAttribute(required = true)
    public String getSystemName()
    {
        return systemName;
    }

    /**
     * Set the database previous package name.
     */
    public void setPreviousSystemName(final String value)
    {
        previousSystemName = value;
    }

    /**
     * Get the database previous package name.
     * 
     * Not added to the XML serialization.
     */
    @XmlTransient
    public String getPreviousSystemName()
    {
        return previousSystemName;
    }

    /**
     * Set the displayed name.
     */
    public void setDisplayName(final String value)
    {
        displayName = value;
    }

    /**
     * Get the displayed name.
     */
    @XmlAttribute(required = true)
    public String getDisplayName()
    {
        return displayName;
    }

    /**
     * Set the name of the rule engine instance on which the rule is executed.
     */
    public void setRuleEngineInstanceName(final String value) {
        ruleEngineInstanceName = value;
    }

    /**
     * Set the name of the rule engine instance on which the rule is executed.
     *
     * Not added to the XML serialization.
     */
    @XmlTransient
    public String getRuleEngineInstanceName() {
        return ruleEngineInstanceName;
    }

    /**
     * Set the state of the package.
     */
    public void setState(final String value)
    {
        state = value;
    }

    /**
     * Get the state of the package.
     *
     * Not added to the XML serialization.
     */
    @XmlTransient
    public String getState()
    {
        return state;
    }

    /**
     * Set the description of the package.
     */
    public void setDescription(final String value)
    {
        description = value;
    }

    /**
     * Set the description of the package.
     */
    @XmlElement(required = true)
    public String getDescription()
    {
        return description;
    }


    // =========================================================================
    // Utils
    // =========================================================================

    /**
     * Returns a String preview of the object.
     */
    @Override
    public String toString()
    {
        return "{displayName:" + displayName +
               ",systemName:" + systemName +
               ",state:" + state + "}";

    }
}
